/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.attachments;

import java.net.URI;
import java.util.Iterator;
import java.util.Set;

import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.transform.Source;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.bean.support.BeanSupport;
import org.apache.servicemix.components.util.ComponentSupport;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.jbi.util.MessageUtil;
/**
 * @author andrea zoppello
 * The AttachmentSplitter split message contents and attachments
 */
public class AttachmentsSplitter extends BeanSupport implements MessageExchangeListener {
	
	private static final Log log = LogFactory.getLog(AttachmentsSplitter.class); 
	
	public static final String SPLITTER_COUNT = "org.apache.servicemix.eip.splitter.count";
    public static final String SPLITTER_INDEX = "org.apache.servicemix.eip.splitter.index";
    public static final String SPLITTER_CORRID = "org.apache.servicemix.eip.splitter.corrid";
	
    public void onMessageExchange(MessageExchange exchange) throws MessagingException {
    	
    		if (exchange instanceof InOnly == false && 
    	            exchange instanceof RobustInOnly == false) {
    	            fail(exchange, new UnsupportedOperationException("Use an InOnly or RobustInOnly MEP"));
    	            return;
    		}
    		if (exchange.getStatus().equals(ExchangeStatus.DONE) ||
    				exchange.getStatus().equals(ExchangeStatus.ERROR)) {
    				// No further processing
    				return;
    		}

    		String processCorrelationId = (String) exchange.getProperty(JbiConstants.CORRELATION_ID);
    		if (processCorrelationId == null) {
    			log.warn("Process Correlation ID not present in exchange: " + exchange);
    			return;
    		}
    		
    		MessageExchange[] parts = createParts(exchange);
    		for (int i = 0; i < parts.length; i++) {

    			parts[i].setProperty(JbiConstants.SENDER_ENDPOINT,
    					getService() + ":" + getEndpoint());

    			parts[i].setProperty(JbiConstants.CORRELATION_ID,
    					processCorrelationId);
    			
    			getDeliveryChannel().send(parts[i]);
    		}
    		
    		done(exchange);
    	
	}
	
	protected MessageExchange[] createParts(MessageExchange exchange) throws MessagingException {
		 NormalizedMessage in = MessageUtil.copyIn(exchange);
	     
		 Set<String> attachmentNames = in.getAttachmentNames();
		 int numberOfAttachments = attachmentNames.size();
	    
	     MessageExchange[] parts = new MessageExchange[1 + numberOfAttachments];
	     parts[0] = createMessageContentPart(exchange.getPattern(), in, in.getContent());
	     
	     int i=0;
	     StringSource aStringSource = null;
	     for (String s : attachmentNames ){
	    	 i++;
	    	 parts[i] = createAttachmentPart(exchange.getPattern(), in, s);
	    	 NormalizedMessage msg = parts[i].getMessage("in");
	    	 msg.setProperty(SPLITTER_COUNT, new Integer(1 + numberOfAttachments));
	         msg.setProperty(SPLITTER_INDEX, new Integer(i));
	         msg.setProperty(SPLITTER_CORRID, exchange.getExchangeId());
	     }
	     return parts;
	}

	
	
	protected MessageExchange createMessageContentPart(URI pattern,
			NormalizedMessage srcMessage, Source content) throws MessagingException {
		
		MessageExchange me = getExchangeFactory().createExchange(pattern);
		NormalizedMessage in = me.createMessage();
		
		in.setContent(content);
		me.setMessage(in, "in");
		
		
			Set names = srcMessage.getPropertyNames();
			for (Iterator iter = names.iterator(); iter.hasNext();) {
				String name = (String) iter.next();
				in.setProperty(name, srcMessage.getProperty(name));
			}
		
		return me;
	}
	
	protected MessageExchange createAttachmentPart(URI pattern,
			NormalizedMessage srcMessage, String attachmentName) throws MessagingException {
		
		MessageExchange me = getExchangeFactory().createExchange(pattern);
		NormalizedMessage in = me.createMessage();
		
	
		in.setContent(new StringSource("<spagic:ATTACHMENT xmlns:spagic=\"urn:it:eng:spagic\" name=\""+attachmentName+"\"></spagic:ATTACHMENT>"));
		me.setMessage(in, "in");
	
		in.addAttachment(attachmentName, srcMessage.getAttachment(attachmentName));
		
			Set names = srcMessage.getPropertyNames();
			for (Iterator iter = names.iterator(); iter.hasNext();) {
				String name = (String) iter.next();
				in.setProperty(name, srcMessage.getProperty(name));
			}
		
		return me;
	}

}
