package org.spagic.smx.components.console;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOptionalOut;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;

import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.listener.MessageExchangeListener;

public class ConsoleOutputComponent implements MessageExchangeListener {
	@Resource
	private DeliveryChannel channel;

	public void onMessageExchange(MessageExchange exchange)
			throws MessagingException {
        // Skip done exchanges
        if (exchange.getStatus() == ExchangeStatus.DONE) {
            return;
        // Handle error exchanges
        } else if (exchange.getStatus() == ExchangeStatus.ERROR) {
            return;
        }
        
        if (isInAndOut(exchange)) {
        	fail(exchange, new UnsupportedOperationException("Use an InOnly or RobustInOnly MEP"));
        } else {
        	try{
        		TransformerFactory tFactory = TransformerFactory.newInstance();
        		Transformer transformer = tFactory.newTransformer();
        		transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        		NormalizedMessage message = exchange.getMessage("in");
        		transformer.transform(message.getContent(), new StreamResult(System.out));
        	}catch (Exception e) {
				e.printStackTrace();
			}
            // Send DONE message
            exchange.setStatus(ExchangeStatus.DONE);
            channel.send(exchange);
        }
    }

    /**
     * A helper method which will return true if the exchange is capable of both In and Out such as InOut,
     * InOptionalOut etc.
     *
     * @param exchange
     * @return true if the exchange can handle both input and output
     */
    protected boolean isInAndOut(MessageExchange exchange) {
        return exchange instanceof InOut || exchange instanceof InOptionalOut;
    }
	
    /**
     * A helper method which fails and completes the given exchange with the specified error
     * @throws MessagingException 
     */
    public void fail(MessageExchange exchange, Exception error) throws MessagingException {
        if (exchange instanceof InOnly || !(error instanceof FaultException)) {
            exchange.setError(error);
        } else {
            FaultException faultException = (FaultException) error;
            exchange.setFault(faultException.getFault());
        }
        channel.send(exchange);
    }

}

