/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.jdbcquery;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.sql.DataSource;

import org.springframework.jdbc.CannotGetJdbcConnectionException;
import org.springframework.jdbc.datasource.DataSourceUtils;

/**
 * Jdbc connection configuration able to initiate the connection itself
 * 
 * @author buso
 *
 */
public class JDBCConnectionConfig {
	
	//
	// Very important for stored procedure
	//
	public static String MS_SQL_SERVER = "MS_SQL";
	public static String MySQL_SERVER = "MySQL";
	public static String Oracle_SERVER = "Oracle";
	public static String Sybase_SERVER = "Sybase";
	public static String DB2_SERVER = "DB2";
	public static String Postgre_SQL_SERVER = "Postgre_SQL";	

	private String driver;
	private String jdbcURL;
	private String userName;
	private String userPassword;
	/**
	 * Usefull if we need to write code specific to a vendor
	 */
	private String databaseVendor;
	
	private DataSource datasource;
	
	public DataSource getDatasource() {
		return datasource;
	}
	public void setDatasource(DataSource datasource) {
		this.datasource = datasource;
	}
	public String getDriver() {
		return driver;
	}
	public void setDriver(String driver) {
		this.driver = driver;
	}
	public String getJdbcURL() {
		return jdbcURL;
	}
	public void setJdbcURL(String jdbcURL) {
		this.jdbcURL = jdbcURL;
	}
	public String getUserName() {
		return userName;
	}
	public void setUserName(String userName) {
		this.userName = userName;
	}
	public String getUserPassword() {
		return userPassword;
	}
	public void setUserPassword(String userPassword) {
		this.userPassword = userPassword;
	}
	
	public Connection getConnection() throws SQLException, ClassNotFoundException, CannotGetJdbcConnectionException {
		//
		// If we're specifying all db parameters get connection using Driver Manager
		//
		if (datasource == null) {
			Class.forName(getDriver());
			
			Connection conn = DriverManager.getConnection(
					getJdbcURL(),
					getUserName(),
					getUserPassword());
			
			return conn;
		}
		//
		// Use Jndi Datasource
		//
		return DataSourceUtils.getConnection(datasource);
		
	}
	public String getDatabaseVendor() {
		return databaseVendor;
	}
	public void setDatabaseVendor(String databaseVendor) {
		this.databaseVendor = databaseVendor;
	}
}
