/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.jdbcstore;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.dbutils.DbUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.bean.support.TransformBeanSupport;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.jbi.util.MessageUtil;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.dom.DOMDocument;
import org.spagic.smx.components.jdbcquery.JDBCConnectionConfig;
import org.spagic.smx.components.jdbcquery.JDBCQueryConfig;
import org.spagic.smx.components.jdbcquery.QueryParameterConfig;
import org.spagic.smx.components.jdbcquery.XmlResultSetHandler;

/**
 * Jdbc component that use the input NormalizedMessage to fill the store
 * procedure call statement.
 * <br>
 * The result is optionally inserted into a copy of the incoming message
 * otherwise a new message is create as output. The output should be formed
 * of more results in this case the call statement placeHolders are used
 * to rappresents output values on output message.
 * 
 * @author buso
 *
 */
public class JDBCStoreProcedureComponent extends TransformBeanSupport {
	
	private static final Log log = LogFactory.getLog(JDBCStoreProcedureComponent.class);
	
	private JDBCConnectionConfig connConfig;
	private JDBCQueryConfig queryConfig;

	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in,
			NormalizedMessage out) throws Exception {

		// transform query in jdbc format
		List<String> orderedParameterNameList = getStringParameterNames(queryConfig.getQuery(), queryConfig.getMatchParamRegexp());
		
		String jdbcQuery = queryConfig.getQuery().replaceAll(queryConfig.getMatchParamRegexp(),"?");
		
		List<QueryParameterConfig> queryParametersConfig = queryConfig.getQueryParams();
		
		
		Object[] results = null;
		Connection conn = null;
		try {
				conn = connConfig.getConnection();
				NewStoreProcedureRunner spRunner = new NewStoreProcedureRunner();
				results = (Object[])spRunner.call(conn, 
												  jdbcQuery, 
												  in, 
												  queryParametersConfig,
												  orderedParameterNameList,
												  new XmlResultSetHandler(
														  	queryConfig.getRowsXmlEnvelope(),
														  	queryConfig.getRowXmlEnvelope())
					);
		} catch (SQLException sqle) {
			if (JDBCQueryConfig.FAULT_FLOW.equals(queryConfig.getFaultManagement())) {
				Fault fault = exchange.createFault();
					Element faultElem = DocumentHelper.createElement("Fault");
					Element sqlStateElem = DocumentHelper.createElement("sql-state");
					sqlStateElem.setText(sqle.getSQLState());
					Element errorCodeElem = DocumentHelper.createElement("error-code");
					errorCodeElem.setText(""+sqle.getErrorCode());
				faultElem.add(sqlStateElem);
				faultElem.add(errorCodeElem);
				fault.setContent(new StringSource(faultElem.asXML()));
				MessageUtil.transfer(fault, out);
				return true;
			}
			throw sqle;
		} finally {
			DbUtils.close(conn);
		}
		
		//
		// compose out response. you need to manage single values and resultset
		// values (last ones were manages by ResultSetHandler)
		//
		MessageUtil.transfer(in, out);
		SourceTransformer trans = new SourceTransformer();
		String strContent = trans.contentToString(out);
		Document docContent = DocumentHelper.parseText(strContent);
		Node contentEnvelop = null;
		
		if (isValidStr(queryConfig.getXmlEnvelope())){
			contentEnvelop = docContent.selectSingleNode(
					"/"+queryConfig.getXmlEnvelope());
			if (contentEnvelop == null) {
				log.warn("Envelope to enrich not found ["+"/"+queryConfig.getXmlEnvelope()+"]: " +
				"the result will be the original message");
			}
		}else{
			log.info("Xml Envelope is null we will keep the original message ");
		}
		
		if (queryConfig.isEnrichMessage() && (contentEnvelop != null)) {
			
			
			if (queryConfig.isEnrichMessage() &&  results.length < 2) {
				Element tmpResult = DocumentHelper.createElement("tmpResults");
				getResultsNodes(results, tmpResult);
				((Element)contentEnvelop).appendContent( tmpResult );
			} else {
				getResultsNodes(results, (Element)contentEnvelop);
			}
			out.setContent(new StringSource( docContent.asXML() ));
		} if (queryConfig.isEnrichMessage() && (contentEnvelop == null)) {
			out.setContent(new StringSource( docContent.asXML() ));
		} else {
			docContent = new DOMDocument();
			String resultEnvelop = queryConfig.getXmlEnvelope();
			if (!isValidStr(resultEnvelop)){
				resultEnvelop = "ENVELOPE";
			}
			docContent.setRootElement(DocumentHelper.createElement(resultEnvelop));
			getResultsNodes(results, docContent.getRootElement());
			out.setContent(new StringSource(docContent.asXML()));
		}
		
		
		return true;
	}
	
	
	
	/**
	 * Return all results in dom4j elements where the element name is the
	 * placeHolder configured for the output parameter.
	 * 
	 * @param results
	 * @param parent
	 */
	private void getResultsNodes(Object[] results, Element parent) {
		Object elemObj = null;
		for (int i = 0; i < results.length; i++) {
			elemObj = results[i];
			if (elemObj instanceof Element) {
				parent.add((Element)elemObj);
			} else {
				Element elem = DocumentHelper.createElement(
					queryConfig.getQueryParams().get(i).getPlaceHolder()
				);
				elem.setText(elemObj != null? elemObj.toString() : "");
				parent.add(elem);
			}
		}
	}
	
	

	public JDBCConnectionConfig getConnConfig() {
		return connConfig;
	}

	public void setConnConfig(JDBCConnectionConfig connConfig) {
		this.connConfig = connConfig;
	}

	public JDBCQueryConfig getQueryConfig() {
		return queryConfig;
	}

	public void setQueryConfig(JDBCQueryConfig queryConfig) {
		this.queryConfig = queryConfig;
	}

	public List<String> getStringParameterNames(String propertyValue, String matchRegexp) {
        Pattern pattern = Pattern.compile(matchRegexp);
        Matcher matcher = 
        	pattern.matcher(propertyValue);
        String placeHolder;
        List<String> toReturn = new ArrayList<String>();
		for (int idx = 1; matcher.find(); idx++) {
			placeHolder = matcher.group();
			placeHolder = placeHolder.substring(1);
			toReturn.add(placeHolder);
		}
		return toReturn;
	}
	
	private boolean isValidStr(String s){
		return s != null && s.trim().length() > 0;
	}

}
