/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.talend;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Set;

import javax.jbi.JBIException;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.Source;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.bean.support.TransformBeanSupport;
import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;

public class TalendCaller extends TransformBeanSupport {

	private static final Log log = LogFactory.getLog(TalendCaller.class);
	private Class talendJobClass; 
	private String  talendJobClassName;
	private String  talendJobClassDescription;
	private HashMap parameters;
	private Method runJob;
	
//	public static Map<String, String> namespaceURI = null;
	
	public TalendCaller()
    {
        talendJobClass = null;
        talendJobClassName = null;
        talendJobClassDescription = null;
        parameters = null;
        runJob = null;
    }

	
	public void start() throws JBIException {
    	
		try{
			log.debug("Talend Caller::start() -> Starting Taleb Caller with Job Class ["+talendJobClassName+"]");
			this.talendJobClass = Class.forName(talendJobClassName);
//			this.runJob = this.talendJobClass.getMethod("runJob", new Class[]{String[].class});
			this.runJob = this.talendJobClass.getMethod("runJobInTOS", new Class[]{String[].class});
			log.debug("Talend Caller::start() -> Talend Job ["+talendJobClassName+"] started ");
//			if (namespaceURI == null)
//				initNameSpaceURI();
			
		}catch (Exception e) {
			log.error("Talend Caller::start() ->   Transformer Class Istantiation Failed");
			log.error(e);
			throw new JBIException(e.getMessage());
		}
    }
	
//	public synchronized static void initNameSpaceURI(){
//		Session aSession = null;
//		try {
//			namespaceURI = new HashMap<String, String>();
//			aSession = HibernateUtil.getSessionFactory().openSession();
//			List<XmlNamespace> lNs = aSession.createQuery(" from org.spagic.metadb.model.XmlNamespace").list();
//			for (XmlNamespace ns : lNs) {
//				namespaceURI.put(ns.getPrefix(), ns.getValue());
//			}
//		}catch(Throwable t){
//			t.printStackTrace();
//		}finally{
//			aSession.close();
//		}
//	}
	
//	public static Map<String, String> getNamespaceURI(){
//		return namespaceURI;
//	}
	
	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in,
			NormalizedMessage out) throws Exception {
		try{	
			SourceTransformer sourceTransformer = new SourceTransformer();
	        String flussoXMLInput = sourceTransformer.contentToString(in); 
	        
			Object[] mainMethodParams  = new Object[1];
			String[] args = new String[parameters.size()];
			Set<String> keySet = parameters.keySet();
			int index = 0;
			Document document = null;
			XPath xPathExpression = null;
			for(String key:keySet) {
				if(!((String)parameters.get(key)).startsWith("/"))
					args[index++] = "--context_param " + key + "=" + parameters.get(key);
				else {
					document = DocumentHelper.parseText(flussoXMLInput);
					xPathExpression = new DefaultXPath((String)parameters.get(key));
					//xPathExpression.setNamespaceURIs(getNamespaceURI());
					String extractedData = xPathExpression.valueOf(document);
					if (extractedData != null && !extractedData.equals(""))
						args[index++] = "--context_param " + key + "=" + extractedData;
				}
			}
			log.debug("Talend Caller::transform() -> prepare param...");
			mainMethodParams[0] = args;
 			int status = ((Integer)runJob.invoke(null, mainMethodParams)).intValue();
 			log.debug("Talend Caller::transform() -> after invoke...");
            log.debug((new StringBuilder("Talend Caller::transform() -> status is: ")).append(status).toString());

            if(status == 0) {
                Source outMsg = null;
                outMsg = toXMLSource("<OK>Talend Caller is OK </OK>");
                out.setContent(outMsg);
            } else {
                Fault fault = exchange.createFault();
                if(status == 1)
                    fault.setContent(new StringSource("<ERROR><TALEND_EXCEPTION/></ERROR>"));
                else
                    fault.setContent(new StringSource((new StringBuilder("<ERROR><TALEND_CODE_ERROR>")).append(status).append("</TALEND_CODE_ERROR></ERROR>").toString()));
                fault.setProperty("org.apache.servicemix.xml", in.getContent());
                throw new FaultException("Talend Caller Failed ", exchange, fault);
            }
		 } catch(Throwable t) {
            log.error("Exception in Talend Caller");
            log.error(t);
            Fault fault = exchange.createFault();
            fault.setContent(new StringSource("<ERROR>An error is occured in Talend Caller</ERROR>"));
            fault.setProperty("org.apache.servicemix.xml", in.getContent());
            throw new FaultException("Talend Caller Failed ", exchange, fault);
        }

		return true;
	}
	
	protected Source toXMLSource(String s) throws Exception {
		return new StringSource(s);
    }

	public String getTalendJobClassName() {
		return talendJobClassName;
	}

	public void setTalendJobClassName(String talendJobClassName) {
		this.talendJobClassName = talendJobClassName;
	}

	public String getTalendJobClassDescription() {
		return talendJobClassDescription;
	}

	public void setTalendJobClassDescription(String talendJobClassDescription) {
		this.talendJobClassDescription = talendJobClassDescription;
	}

	public HashMap getParameters() {
		return parameters;
	}

	public void setParameters(HashMap parameters) {
		this.parameters = parameters;
	}
	
    
	
}
