package org.spagic.smx.components.velocity;

import java.io.IOException;
import java.io.StringWriter;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import javax.annotation.PostConstruct;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.bean.support.TransformBeanSupport;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.app.event.EventCartridge;
import org.apache.velocity.runtime.RuntimeConstants;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.springframework.core.io.Resource;

public class VelocityTemplateMerger extends TransformBeanSupport {
	
	private static final Log log = LogFactory.getLog(VelocityTemplateMerger.class);
	
	private Map<String, String> namespaceMap = null;
	
	private static final String OUTPUT_TYPE_NORMALIZED_MESSAGE = "OUTPUT_TYPE_NORMALIZED_MESSAGE";
	private static final String OUTPUT_TYPE_MESSAGE_HEADER = "OUTPUT_TYPE_MESSAGE_HEADER";
	
	private Properties mappings = null;
	
	// Determine if the output will be the normalized Message or a message Header Properties
	
	private String velocityTemplateName = null;
	private String outputType = null;
	
	// To use only if outputType = OUTPUT_TYPE_MESSAGE_HEADER
	private String outputPropertyName = null;
	
	private Resource contextMappingFile = null;
	
	@PostConstruct
	public void init() {
		  log.info(" Initializing VelocityTemplateMerger ");
		  mappings = new Properties();
		  try{
			  if (contextMappingFile != null){ 
				  mappings.load(contextMappingFile.getInputStream());
			  }else{
				  log.warn(" No Context Mapping File Found ");
			  }
		  }catch (IOException e) {
			  log.error(" An Error has occurred during initialization ", e);
		  }
	      log.info(" Initialization OK ");
	}
	
	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in,
			NormalizedMessage out) throws Exception {
		synchronized (this) {
			
		
			log.info("Velocity ["+Thread.currentThread().getId()+"] | Start");
			log.debug(" transform -> Configure Velocity Context");
			VelocityContext velocityContext = configureVelocityContext(in);
			StringWriter sw = new StringWriter();
    	
			log.debug(" transform -> Perform Velocity Template Merge");
		
    		Velocity.setProperty(RuntimeConstants.RESOURCE_LOADER, "cp");;
    		Velocity.setProperty("cp.resource.loader.description","Velocity Classpath Resource Loader");
    		Velocity.setProperty("cp.resource.loader.class","org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader");
    	
    		Velocity.mergeTemplate(velocityTemplateName, velocityContext, sw );
    	
    		if (outputType.equalsIgnoreCase(OUTPUT_TYPE_NORMALIZED_MESSAGE)){
    			out.setContent(new StringSource(sw.toString()));
    		}else{		
    			out.setProperty(outputPropertyName, sw.toString());
    		}
    		log.info("Velocity ["+Thread.currentThread().getId()+"] | End");
		}
		return true;
	}
	
	public VelocityContext configureVelocityContext(NormalizedMessage in) throws Exception{
		
		String inputMessageAsXML = new SourceTransformer().contentToString(in);
		Document inputMessageAsDocument = DocumentHelper.parseText(inputMessageAsXML);
		
		VelocityContext context = new VelocityContext();
		
    	EventCartridge ec = new EventCartridge();
    	ec.addInvalidReferenceEventHandler(new ReturnNullForInvalidReferences());
    	ec.attachToContext(context);
    	
    	String rule = null;
    	String value = null;
    	
    	// rule are in the form 
    	// 1) contextVarName = xpath->xpathExpression
    	// 2) contextVarName = property->msgProperty
    	
    	for ( Object key : mappings.keySet()){
    		rule = mappings.getProperty((String)key);
    		value = null;
    		if (rule.startsWith("xpath")){
    			value = evalXPath(stripRule(rule),inputMessageAsDocument);
    		}else{
    			value = String.valueOf(in.getProperty(stripRule(rule)));
    		}
    		log.info(" Puuting on Context ["+((String)key)+"] -> ["+value+"]");
    		context.put((String)key, value);
    	}
    	
    	return context;

	}
	
	protected String stripRule(String rule){
		log.debug(" Rule is " + rule);
		String stripped =  rule.substring(rule.indexOf("->")+2).trim();
		log.debug(" Stripped Rule is " + stripped);
		return stripped;
	}
	
	protected String evalXPath(String xpath, Document doc){
		XPath xPathExpression = new DefaultXPath(xpath);
		xPathExpression.setNamespaceURIs(getNamespaceMap());
		return xPathExpression.valueOf(doc);
	}
	
	public String getOutputType() {
		return outputType;
	}

	public void setOutputType(String outputType) {
		this.outputType = outputType;
	}

	public Map<String, String> getNamespaceMap() {
		return namespaceMap;
	}

	public void setNamespaceMap(Map<String, String> namespaceMap) {
		this.namespaceMap = namespaceMap;
	}

	public String getVelocityTemplateName() {
		return velocityTemplateName;
	}

	public void setVelocityTemplateName(String velocityTemplateName) {
		this.velocityTemplateName = velocityTemplateName;
	}

	public String getOutputPropertyName() {
		return outputPropertyName;
	}

	public void setOutputPropertyName(String outputPropertyName) {
		this.outputPropertyName = outputPropertyName;
	}
	
	public void setContextMappingFile(Resource contextMappingFile) {
	      this.contextMappingFile = contextMappingFile;
	}
	

}
