package org.spagic.smx.components.wait;
    
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.List;
import java.util.Map;

import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.jbi.messaging.RobustInOnly;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.bean.support.BeanSupport;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.hibernate.Query;
import org.hibernate.Session;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.spagic.event.model.BaseEvent;
import org.spagic.event.model.DateUtils;
import org.spagic.event.model.FileEvent;
import org.spagic.event.model.IEvent;
import org.spagic.metadb.model.WaitData;
import org.spagic.metadb.model.WaitDataAttribute;

public abstract class EventBeanComponent extends BeanSupport {
	
	private static final Log log = LogFactory.getLog(EventBeanComponent.class);

	private Map<String, String> namespaceMap = null;
	
	protected static final String EXPIRATION_POLICY_NEVER_EXPIRE="NeverExpire";
	protected static final String EXPIRATION_POLICY_AT_FIXED_DATE_TIME="AtFixedDateTime";
	protected static final String EXPIRATION_POLICY_AFTER_DATE_TIME_INTERVAL="AfterTimeInterval";
	
	protected static final String UNIT_INTERVAL_DAY = "day";
	protected static final String UNIT_INTERVAL_MONTH = "month";
	protected static final String UNIT_INTERVAL_YEAR = "year";
	
	protected static final String UNIT_INTERVAL_HOUR = "hour";
	protected static final String UNIT_INTERVAL_MINUTE = "minute";
	protected static final String UNIT_INTERVAL_SECOND = "second";
	
	protected static final String ON_EXPIRATION_RESUME_PROCESS = "ResumeProcess";
	protected static final String ON_EXPIRATION_KIL_PROCESS =    "KillProcess";
	
	
	// JODA_DATE_TIME_FORMATTER
	protected static DateTimeFormatter dateTimefmt = DateTimeFormat.forPattern("dd-MM-yyyy HH:mm:ss");
	
	/**
	 * Delete from the database the row relative to wait_data_id
	 * @param session
	 * @param wd
	 */
	protected void deleteWaitData(Session session, WaitData wd){
		
		
		String hql = "from org.spagic.metadb.model.WaitDataAttribute wda where wda.waitData.idWait = :pIdWaitData ";
		Query aQuery = session.createQuery(hql);
		aQuery.setLong("pIdWaitData", wd.getIdWait());
		
	
		List<WaitDataAttribute> wdAttributes =  aQuery.list();
		for (WaitDataAttribute wda : wdAttributes){
			session.delete(wda);
			session.flush();
		}
		
		session.delete(wd);
		session.flush();
	}
	
	public Map<String, String> getNamespaceMap() {
		return namespaceMap;
	}

	public void setNamespaceMap(Map<String, String> namespaceMap) {
		this.namespaceMap = namespaceMap;
	}
	
	public IEvent getEventFromExchange(MessageExchange exchange) throws Exception {
		NormalizedMessage inMessage = exchange.getMessage("in");
		String eventAsXml = new SourceTransformer().toString(inMessage.getContent());		
		
		IEvent ev = null;
		// String eventType 
		Document eventAsDocument = DocumentHelper.parseText(eventAsXml);
				
		String eventType = evalXPathXmlAttribute(eventAsDocument,  "/spagicevent:GenericEvent/@eventType");
		
		if (eventType.equalsIgnoreCase(IEvent.FILE_EVENT_TYPE)){
			ev = new FileEvent();
		}else{
			ev = new BaseEvent();
			ev.setEventType(eventType);
		}
		
		String eventDate = evalXPathXmlAttribute(eventAsDocument,  "/spagicevent:GenericEvent/@date");
		String eventTimestamp = evalXPathXmlAttribute(eventAsDocument,  "/spagicevent:GenericEvent/@timeStamp");
		String evSignalEndpointURI = evalXPathXmlAttribute(eventAsDocument,  "/spagicevent:GenericEvent/@signalEndpointURI");
		
		
		if (eventDate != null && eventDate.trim().length() > 0){
			ev.setDate(DateUtils.parseDate(eventDate));
		}
		
		if (eventTimestamp != null && eventTimestamp.trim().length() > 0){
			ev.setTimeStamp(DateUtils.parseDate(eventTimestamp));
		}
		
		if (evSignalEndpointURI != null && evSignalEndpointURI.trim().length() > 0){
			ev.setSignalEndpointURI(QName.valueOf(evSignalEndpointURI));
		}
		
		
		Node payloadNode = eventAsDocument.selectSingleNode("/spagicevent:GenericEvent/spagicevent:payload");
		String payload = payloadNode.asXML();
		String realPayload = payload.substring(payload.indexOf(">") + 1);
		realPayload = realPayload.substring(0, realPayload.indexOf("</spagicevent:"));
		
		ev.setXmlPayload(realPayload);
		
		List<Node> eventAttributes = eventAsDocument.selectNodes("/spagicevent:GenericEvent/spagicevent:attribute");
		
		for (Node n : eventAttributes){
			ev.setEventAttribute(n.valueOf("@name"), n.valueOf("@value"));
		}
		
		return ev;
	}
	
	public String evalXPathXmlAttribute( Document doc, String xPathExpressionString){
		XPath xPathExpression = new DefaultXPath(xPathExpressionString);
		xPathExpression.setNamespaceURIs(getNamespaceMap());
		return xPathExpression.valueOf(doc);
	}
	
	public WaitData readWaitData(Session session, String waitId){
		
		   WaitData wd =(WaitData) session.get(WaitData.class, Long.valueOf(waitId));
		   wd.getExchange();
		   return wd;
	}
	
	
	protected MessageExchange deserializeExchange(byte[] buf) {
		MessageExchange exchange = null;
		
		if (buf == null) {
			return null;
		}
		try {
			// Deserialize from a byte array
			ObjectInputStream in = new ObjectInputStream(new ByteArrayInputStream(buf));
			exchange = (MessageExchange) in.readObject();
	        in.close();
		} catch (Exception ex) {
			log.error("Error serializing exchange " + exchange, ex );
		}
        return exchange;
	}
	
	protected byte[] serializeExchange(MessageExchange exchange) {
		byte[] buf = null;

		// Serialize to a byte array
		try {
			ByteArrayOutputStream bos = new ByteArrayOutputStream();
			ObjectOutputStream out = new ObjectOutputStream(bos);
			out.writeObject(exchange);
			out.close();

			// Get the bytes of the serialized object
			buf = bos.toByteArray();
		} catch (Exception ex) {
			log.error("Error serializing exchange " + exchange, ex);
		}
		return buf;
	}
	
	protected boolean isValidString(String str){
		return ( str != null ) && ( str.trim().length() > 0 );
	}
	
	protected void populateMessage(NormalizedMessage nm, String xmlPayload) throws MessagingException {
		if (xmlPayload != null) {	
			nm.setContent(new StringSource(xmlPayload));
		} else {
			nm.setContent( new StringSource("<SIGNAL></SIGNAL>"));
		}	
	}
	
	 protected boolean isRobustInOnly(MessageExchange exchange) {
			return exchange instanceof RobustInOnly;
		}

		protected boolean isInOnly(MessageExchange exchange) {
			return exchange instanceof InOnly;
		}
}
