package org.spagic.smx.components.wait;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.model.ProcessInstance;
import org.spagic.metadb.model.WaitData;

public class EventJobComponent extends EventBeanComponent implements MessageExchangeListener {
	
	private static final Log log = LogFactory.getLog(EventJobComponent.class);
	
	private SimpleDateFormat quartzDateTimeFormatter = new SimpleDateFormat("EEE MMM dd HH:mm:ss z yyyy",Locale.US);
	
	
	private SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss z");
	@Resource
	private DeliveryChannel channel;
	
	
	public void onMessageExchange(MessageExchange exchange) throws MessagingException {
		
		// Skip done exchanges
		if (exchange.getStatus() == ExchangeStatus.DONE) {
			return;
			// Handle error exchanges
		} else if (exchange.getStatus() == ExchangeStatus.ERROR) {
				return;
		}
		try{
			NormalizedMessage in = exchange.getMessage("in");
			String normalizedMessageContent = new SourceTransformer().contentToString(in);
			
			Date dateFromChron = getFireEventDateFromMessage(normalizedMessageContent);
			
			if (dateFromChron != null){
				timedJob(dateFromChron);
				if (isInAndOut(exchange))
					responseOk(exchange);
				else
					done(exchange);
			}else{
				killInstanceJob(getWaitId(normalizedMessageContent));
				if (isInAndOut(exchange))
					responseOkForKill(exchange);
				else
					done(exchange);
			}
			
			
			
		}catch (Exception e) {
			try{
				fail(exchange, e);
			} catch (Exception e2) {
				log.warn("Unable to handle error: " + e2, e2);
				if (logger.isDebugEnabled()) {
					log.debug("Original error: " + e, e);
				}
			}
		}
	}

	protected Date getFireEventDateFromMessage(String xmlContent) throws Exception {
		Document doc = DocumentHelper.parseText(xmlContent);
		Node nFireTime = doc.selectSingleNode("/timer/fireTime");
		
		if (nFireTime != null){
			return quartzDateTimeFormatter.parse(nFireTime.getText());
		}else{
			return null;
		}
	}
	
	protected String getWaitId(String xmlContent) throws Exception {
		Document doc = DocumentHelper.parseText(xmlContent);
		return doc.valueOf("/parameters/@waitId");
	}
	
	public void responseOk(MessageExchange exchange) throws MessagingException {
		NormalizedMessage out = exchange.createMessage();
		String xmlPayload = "<RESPONSE>OK</RESPONSE>";
		out.setContent(new StringSource(xmlPayload));
		exchange.setMessage(out, "out");
		channel.send(exchange);
	}
	
	public void responseOkForKill(MessageExchange exchange) throws MessagingException {
		NormalizedMessage out = exchange.createMessage();
		String xmlPayload = "<spagickill:response xmlns:spagickill=\"urn:it:eng:spagic:kill\">OK</spagickill:response>";
		out.setContent(new StringSource(xmlPayload));
		exchange.setMessage(out, "out");
		channel.send(exchange);
	}
	public void killInstanceJob(String waitId) throws Exception {
		log.info("killInstanceJob - Start");
		Session aSession = null;
		Transaction tx = null;
		try {
			log.info("killInstanceJob - Getting Session from Hibernate");
			aSession = HibernateUtil.getSessionFactory().openSession();
			tx = aSession.beginTransaction();
			WaitData wd = readWaitData(aSession, waitId);
			ProcessInstance pi = AuditDBUtils.getProcessInstanceByIdCorrelation(aSession, wd.getIdCorrelation());
			if (pi != null)
				pi.setState(ProcessInstanceStateConstants.PROCESS_KILLED);
			
			deleteWaitData(aSession, wd);
			tx.commit();
		} catch (Throwable e) {
			log.error("killInstanceJob - Has Produced an exception", e);
			log.error("killInstanceJob  - Rollback transition");
			if (tx != null)
				tx.rollback();

		} finally {
			
			log.info("deleteInstanceJob - Close Session");
			if ((aSession != null) && aSession.isOpen()) {
				aSession.close();
			}
			
			log.info("deleteInstanceJob - End");
		}
	}
		
	public void timedJob(Date now) throws Exception {
		log.info("timedJob - Start");
		Session aSession = null;
		Transaction tx = null;
		try {
			log.info("timedJob - Getting Session from Hibernate");
			aSession = HibernateUtil.getSessionFactory().openSession();
			tx = aSession.beginTransaction();
			
			String hql = "from WaitData wd where wd.expiryTimestamp < :par1";
			Query query = aSession.createQuery(hql);
			query.setTimestamp("par1", now);
			//System.out.println("Now is ["+simpleDateFormat.format(now)+"]");
			List<WaitData> expiredProcessList = query.list();
			log.info("timedJob -Founded "+ expiredProcessList.size() + " Expired Process");
			for (WaitData wd : expiredProcessList){
				if (wd.getExpirationAction().equalsIgnoreCase(ON_EXPIRATION_RESUME_PROCESS)){
					resumeProcessInstance(aSession, wd);
				}else if (wd.getExpirationAction().equalsIgnoreCase(ON_EXPIRATION_KIL_PROCESS)){
					ProcessInstance pi = AuditDBUtils.getProcessInstanceByIdCorrelation(aSession, wd.getIdCorrelation());
					if (pi != null)
						pi.setState(ProcessInstanceStateConstants.PROCESS_KILLED);
					deleteWaitData(aSession, wd);
				}else{
					ProcessInstance pi = AuditDBUtils.getProcessInstanceByIdCorrelation(aSession, wd.getIdCorrelation());
					if (pi != null)
						pi.setState(ProcessInstanceStateConstants.PROCESS_EXPIRED);
					deleteWaitData(aSession, wd);
				}
			}
			tx.commit();
		} catch (Throwable e) {
			log.error("timedJob - Has Produced an exception", e);
			log.error("timedJob  - Rollback transition");
			if (tx != null)
				tx.rollback();

		} finally {
			
			log.info("timedJob - Close Session");
			if ((aSession != null) && aSession.isOpen()) {
				aSession.close();
			}
			
			log.info("timedJob - End");
		}
	}
	
	protected void resumeProcessInstance(Session session, WaitData wd) throws Exception {
		MessageExchange readedExchange = deserializeExchange(wd.getExchange());
		String signalEndpointURI = wd.getSignalUri();		
		QName serviceSignalEndpointQName = QName.valueOf(signalEndpointURI);
		
		log.info("GenericSignalInvoker->receiveSignal - Event is arrived with no payload use the freezed messages in wait as signal content");
		NormalizedMessage readedMessage = readedExchange.getMessage("in");
		String newMessageContent = new SourceTransformer().contentToString(readedMessage);
		
		MessageExchange newExchange = null;
		if (isInOnly(readedExchange)) {
			log.info("SignalComponent->signal - Retrieved Exchange was an InOnly -> Need to Create another exchange");
			newExchange = channel.createExchangeFactory().createInOnlyExchange();
		}else if (isRobustInOnly(readedExchange)){
			log.info("SignalComponent->signal - Retrieved Exchange was an RobustInOnly -> Need to Create another exchange");
			newExchange = channel.createExchangeFactory().createRobustInOnlyExchange();
		}
		
		NormalizedMessage newMessage = newExchange.createMessage();
		populateMessage(newMessage, newMessageContent);
		newExchange.setService(serviceSignalEndpointQName);
		newExchange.setMessage(newMessage, "in");
		
		log.info("SignalComponent->signal - InOnly send");
		channel.send(newExchange);
		
		deleteWaitData(session, wd);

	}

	
		
	
		
}
