package org.spagic.smx.components.wait;

import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.spagic.event.model.IEvent;
import org.spagic.event.model.IMatchingIstanceRuleOnWaitData;
import org.spagic.event.model.InstanceMatcher;
import org.spagic.event.model.WaitingProcessWrapper;
import org.spagic.metadb.dbutils.HibernateUtil;


public class EventMatcherComponent extends EventBeanComponent implements MessageExchangeListener {

	private static final Log log = LogFactory.getLog(EventMatcherComponent.class);
	private QName signalEndpointURI = null;
	
	private List<IMatchingIstanceRuleOnWaitData> matchingRulesOnEventAttribute = null;
	private List<IMatchingIstanceRuleOnWaitData> matchingRulesOnEventPayload = null;
	
	
	@Resource
	private DeliveryChannel channel;

	public void onMessageExchange(MessageExchange exchange)
			throws MessagingException {
		if (channel == null){
			log.warn(" **************** Channel is null **************** ");
		}
		if (exchange == null) {
			return;
		}
		// Skip done exchanges
		if (exchange.getStatus() == ExchangeStatus.DONE) {
			return;
			// Handle error exchanges
		} else if (exchange.getStatus() == ExchangeStatus.ERROR) {
			return;
		}
		try{
			matchEvents(exchange);
			done(exchange);
		}catch (Exception e) {
			try{
				fail(exchange, e);
			} catch (Exception e2) {
				log.warn("Unable to handle error: " + e2, e2);
				if (logger.isDebugEnabled()) {
					log.debug("Original error: " + e, e);
				}
			}
			
		}
			
	
	}

	/**
	 * handles the incoming provider messages
	 * 
	 * @param exchange
	 * @throws MessagingException
	 */
	private void matchEvents(MessageExchange exchange) throws Exception {
		// Retrieve correlation id
		Session session = null;
		Transaction tx = null;
		try{
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			log.info("EventMatcherComponent->wait - Start");
			InstanceMatcher instanceMatcher = new InstanceMatcher();
			instanceMatcher.setNamespaces(getNamespaceMap());
				List<IMatchingIstanceRuleOnWaitData> rules = new ArrayList<IMatchingIstanceRuleOnWaitData>();
			if (matchingRulesOnEventAttribute != null)
				rules.addAll(matchingRulesOnEventAttribute);
			if (matchingRulesOnEventPayload != null)
				rules.addAll(matchingRulesOnEventPayload);
			IEvent ev = getEventFromExchange(exchange);
			List<WaitingProcessWrapper> macthingProcesses = null;
			try{
			if (ev == null){
				System.out.println("  Ev is NULL ");
			}
			if (this.signalEndpointURI == null){
				System.out.println("  signalEndpointURI is NULL ");
			}
			if (rules == null){
				System.out.println("  rules is NULL ");
			}
			if (instanceMatcher == null){
				System.out.println("  instanceMatcher is NULL ");
			}
			macthingProcesses = instanceMatcher.getWaitingProcessMatchingEvent(ev, this.signalEndpointURI.toString(), rules);
			}catch (Throwable t) {
				t.printStackTrace();
			}
			
			log.info("EventMatcherComponent->Founded "+macthingProcesses.size() + " macthingProcesses");
			WaitingProcessWrapper candidateProcess = null;
			if (macthingProcesses.size() > 0){
				candidateProcess = macthingProcesses.get(0);
				MessageExchange savedExchange = deserializeExchange(candidateProcess.getExchange());
				
				String signalString = createSignalEventXMLPaylodForEvent(ev, candidateProcess);
			
				MessageExchange newExchange = null;
				newExchange = channel.createExchangeFactory().createInOnlyExchange();
		
			
				NormalizedMessage newMessage = newExchange.createMessage();
				populateMessage(newMessage, signalString);
				newExchange.setService(this.signalEndpointURI);
				newExchange.setMessage(newMessage, "in");
				newExchange.setProperty(JbiConstants.CORRELATION_ID, savedExchange.getProperty(JbiConstants.CORRELATION_ID));
		
				channel.send(newExchange);
				deleteWaitData(session, readWaitData(session, String.valueOf(candidateProcess.getIdWait())));
			
			}
			tx.commit();
		} catch (Throwable e) {
			e.printStackTrace();
			log.error(e);
			if (tx != null && tx.isActive()){
				tx.rollback();
			}
			throw new Exception(e);
		}finally{
	 		if ((session != null) && session.isOpen()) {
	 			   session.close();
	 		}
	 	}
	}
	
	protected void populateMessage(NormalizedMessage nm, String xmlPayload) throws MessagingException {
		if (xmlPayload != null) {	
			nm.setContent(new StringSource(xmlPayload));
		} else {
			nm.setContent( new StringSource("<SIGNAL></SIGNAL>"));
		}	
	}
	
	
	public String createSignalEventXMLPaylodForEvent(IEvent ev, WaitingProcessWrapper candidateProcess ){
		String signalString ="<spagicevent:Signal  xmlns:spagicevent=\"urn:it:eng:spagic:event\" waitId=\""+candidateProcess.getIdWait()+"\" eventType=\""+ev.getEventType()+"\">";
		signalString += ev.asXML();
		signalString +="</spagicevent:Signal>";
		return signalString;
	}
	
	
	public QName getSignalEndpointURI() {
		return signalEndpointURI;
	}

	public void setSignalEndpointURI(QName signalEndpointURI) {
		this.signalEndpointURI = signalEndpointURI;
	}

	public List<IMatchingIstanceRuleOnWaitData> getMatchingRulesOnEventAttribute() {
		return matchingRulesOnEventAttribute;
	}

	public void setMatchingRulesOnEventAttribute(
			List<IMatchingIstanceRuleOnWaitData> matchingRulesOnEventAttribute) {
		this.matchingRulesOnEventAttribute = matchingRulesOnEventAttribute;
	}

	public List<IMatchingIstanceRuleOnWaitData> getMatchingRulesOnEventPayload() {
		return matchingRulesOnEventPayload;
	}

	public void setMatchingRulesOnEventPayload(
			List<IMatchingIstanceRuleOnWaitData> matchingRulesOnEventPayload) {
		this.matchingRulesOnEventPayload = matchingRulesOnEventPayload;
	}
	
}
