package org.spagic.smx.components.wait;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.model.WaitData;

public class GenericSignalInvoker extends EventBeanComponent implements MessageExchangeListener {

	private static final Log log = LogFactory.getLog(GenericSignalInvoker.class);
	
	@Resource
	private DeliveryChannel channel;
	
	
	public void onMessageExchange(MessageExchange exchange)
			throws MessagingException {
		if (exchange.getStatus() == ExchangeStatus.ACTIVE) {
			if ( isInAndOut(exchange) ){
				try{
					receiveSignal(exchange);
					responseOk(exchange);
				}catch (Exception e) {
					responseFault(exchange);
				}
			}else{
				throw new MessagingException(" GenericSignalInvoker Component accept only InOut - Exchange with status Active ");
			}
		}else{
        	return;
        }		
	}

	

	/**
	 * handles the incoming provider messages
	 * 
	 * @param exchange
	 * @throws MessagingException
	 */
	protected void receiveSignal(MessageExchange exchange)
			throws MessagingException {
		
		log.info("GenericSignalInvoker->signal - Start");
		XPath xPathExpression = null;
		Session session = null;
		Transaction tx = null;
		try{
			session = HibernateUtil.getSessionFactory().openSession();
			tx = session.beginTransaction();
			
			NormalizedMessage normalizedMessage = exchange.getMessage("in");
			String xmlString = new SourceTransformer().contentToString(normalizedMessage);
			
			log.info("GenericSignalInvoker->Event Received");
			log.info(xmlString);
			
			// Reading Data
			Document spagicEventDocument = DocumentHelper.parseText(xmlString);
			
			String xPathExpressionForWaitID = "/spagicevent:Signal/@waitId";
			String xPathExpressionForEventType  = "/spagicevent:Signal/@eventType";
			
			xPathExpression = new DefaultXPath(xPathExpressionForWaitID);
			xPathExpression.setNamespaceURIs(getNamespaceMap());
			String waitId  = xPathExpression.valueOf(spagicEventDocument);
			
			xPathExpression = new DefaultXPath(xPathExpressionForEventType);
			xPathExpression.setNamespaceURIs(getNamespaceMap());
			String eventType = xPathExpression.valueOf(spagicEventDocument);
			
			
			
			log.info("GenericSignalInvoker->receiveSignal - Retrieving WaitData WithID and get the Serialized Exchange");
			WaitData readedWd = readWaitData(session,waitId);
			if (readedWd == null){
				log.warn("GenericSignalInvoker->receiveSignal - No WaitData Record retrieved for wait ID");
				tx.rollback();
				return;
			}
			
			MessageExchange readedExchange = deserializeExchange(readedWd.getExchange());
			String signalEndpointURI = readedWd.getSignalUri();		
			QName serviceSignalEndpointQName = QName.valueOf(signalEndpointURI);
			
			// Must see if the event arrived has a payload
			String xPathToSelectPayloadOfEvent = "/spagicevent:Signal/spagicevent:GenericEvent/spagicevent:payload";
			Node eventPayloadNode = null;
			String newMessageContent = null;
			xPathExpression = new DefaultXPath(xPathToSelectPayloadOfEvent);
			xPathExpression.setNamespaceURIs(getNamespaceMap());
			eventPayloadNode = xPathExpression.selectSingleNode(spagicEventDocument);
				
			if  (eventPayloadNode != null){
					log.info("GenericSignalInvoker->receiveSignal - Event is arrived with a payload using this as the new signal message content");
					newMessageContent = eventPayloadNode.asXML();
			}
			
			if ( newMessageContent == null ) {
				log.info("GenericSignalInvoker->receiveSignal - Event is arrived with no payload use the freezed messages in wait as signal content");
				NormalizedMessage readedMessage = readedExchange.getMessage("in");
				newMessageContent = new SourceTransformer().contentToString(readedMessage);
			}
			
			MessageExchange newExchange = null;
			if (isInOnly(readedExchange)) {
				log.info("SignalComponent->signal - Retrieved Exchange was an InOnly -> Need to Create another exchange");
				newExchange = channel.createExchangeFactory().createInOnlyExchange();
				
			}else if (isRobustInOnly(readedExchange)){
				log.info("SignalComponent->signal - Retrieved Exchange was an RobustInOnly -> Need to Create another exchange");
				newExchange = channel.createExchangeFactory().createRobustInOnlyExchange();
			}
			
			newExchange.setProperty(JbiConstants.CORRELATION_ID, readedExchange.getProperty(JbiConstants.CORRELATION_ID));
			newExchange.setProperty(JbiConstants.SENDER_ENDPOINT, readedExchange.getService().toString());
					
			NormalizedMessage newMessage = newExchange.createMessage();
			populateMessage(newMessage, newMessageContent);
			newExchange.setService(serviceSignalEndpointQName);
			newExchange.setMessage(newMessage, "in");
			
			log.info("SignalComponent->signal - InOnly send");
			channel.send(newExchange);
			
			deleteWaitData(session, readedWd);
			tx.commit();
		} catch (Throwable e) {
			log.error(e);
			if (tx != null && tx.isActive()){
				tx.rollback();
			}
			
			//fail(exchange,new Exception(e));
		}finally{
	 		if ((session != null) && session.isOpen()) {
	 			   session.close();
	 		}
	 	}
	}
	
	public void responseOk(MessageExchange exchange) throws MessagingException {
		NormalizedMessage out = exchange.createMessage();
		String xmlPayload = "<spagicevent:SignalResponse xmlns:spagicevent=\"urn:it:eng:spagic:event\">OK</spagicevent:SignalResponse>";
		out.setContent(new StringSource(xmlPayload));
		exchange.setMessage(out, "out");
		channel.send(exchange);
	}
	
	public void responseFault(MessageExchange exchange) throws MessagingException {
		NormalizedMessage out = exchange.createMessage();
		String xmlPayload = "<spagicevent:SignalResponse xmlns:spagicevent=\"urn:it:eng:spagic:event\">FAULT</spagicevent:SignalResponse>";
		out.setContent(new StringSource(xmlPayload));
		exchange.setMessage(out, "out");
		channel.send(exchange);
	}
	
	
	
	
	
	
	
		
	
	
	
	

	

}
