package org.spagic.smx.components.wait;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;

public class SignalComponent extends EventBeanComponent implements MessageExchangeListener  {

	private static final Log log = LogFactory.getLog(SignalComponent.class);
	
	private  QName destinationEndpointURI = null;
	
	@Resource
	private DeliveryChannel channel;

	public void onMessageExchange(MessageExchange exchange)
			throws MessagingException {
		if ( isInOnly(exchange) || isRobustInOnly(exchange) ){
			if (exchange.getStatus() == ExchangeStatus.ACTIVE) {
				try{
					signal(exchange);
					done(exchange);
				}catch (Exception e) {
					try{
						fail(exchange, e);
					} catch (Exception e2) {
						log.warn("Unable to handle error: " + e2, e2);
						if (logger.isDebugEnabled()) {
							log.debug("Original error: " + e, e);
					}
					}
				}
			}else{
				return;
			}
		}else{
        	throw new MessagingException(" GenericSignalInvoker Component Only accept InOnly Exchange ");
        }
		
	}
	

	/**
	 * handles the incoming provider messages
	 * 
	 * @param exchange
	 * @throws MessagingException
	 */
	protected void signal(MessageExchange exchange) throws Exception {
		
		
			log.info("SignalComponent->signal - Start");
		
			NormalizedMessage normalizedMessage = exchange.getMessage("in");
			String xmlString = new SourceTransformer().contentToString(normalizedMessage);
			
			log.info("Signal Message");
			log.info(xmlString);
			
			MessageExchange newExchange = null;
			if (isInOnly(exchange)){
				log.info("SignalComponent->signal - Retrieved Exchange was an InOnly -> Need to Create another exchange");
				newExchange = channel.createExchangeFactory().createInOnlyExchange();
			}else if (isRobustInOnly(exchange)){
				log.info("SignalComponent->signal - Retrieved Exchange was an RobustInOnly -> Need to Create another exchange");
				newExchange = channel.createExchangeFactory().createRobustInOnlyExchange();
			}
			log.info("SignalComponent->signal - Populate in message of new created exchange with signalPayload");
			NormalizedMessage in = newExchange.createMessage();
			in.setContent(normalizedMessage.getContent());
			newExchange.setMessage(in, "in");
			newExchange.setService(destinationEndpointURI);
			log.info("SignalComponent->signal - InOnly send");
			channel.send(newExchange);		
	}



	public QName getDestinationEndpointURI() {
		return destinationEndpointURI;
	}



	public void setDestinationEndpointURI(QName destinationEndpointURI) {
		this.destinationEndpointURI = destinationEndpointURI;
	}

}
