package org.spagic.smx.components.wait;

import java.util.Date;
import java.util.List;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.joda.time.DateTime;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.spagic.event.model.IEvent;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.model.WaitData;
import org.spagic.metadb.model.WaitDataAttribute;

import com.sun.org.apache.bcel.internal.generic.CPInstruction;

public class WaitComponent extends EventBeanComponent implements MessageExchangeListener {

	private static final Log log = LogFactory.getLog(WaitComponent.class);
	
	private String eventType = null;
	private QName signalEndpointURI = null;
	private QName destinationEndpointURI = null;
	private List<WaitDataRule> waitDataRules = null;
	
	// USED IF EXPIRATION POLICY IS EXPIRATION_POLICY_AT_FIXED_DATE_TIME
	private String expirationPolicy = null; 
	private String fixedExpirationDateTime = null; // IN THE FORMAT (DD-MM-YYYY HH24:mm:ss)
	
	
	// USED IF EXPIRATION POLICY IS EXPIRATION_POLICY_AFTER_DATE_TIME_INTERVAL
	private String expirationAfter = null;
	private String intervalDateTimeUnit = null;
	
	private String expirationAction = null;
	
	
	@Resource
	private DeliveryChannel channel;

	public void onMessageExchange(MessageExchange exchange)
			throws MessagingException {
		if (channel == null){
			System.out.println(" **************** Channel is null ****************");
		}
		if (exchange == null) {
			return;
		}
		// Skip done exchanges
		if (exchange.getStatus() == ExchangeStatus.DONE) {
			return;
			// Handle error exchanges
		} else if (exchange.getStatus() == ExchangeStatus.ERROR) {
			return;
		}
		try{
			wait(exchange);
		}catch (Exception e) {
			try{
				fail(exchange, e);
			} catch (Exception e2) {
				log.warn("Unable to handle error: " + e2, e2);
				if (logger.isDebugEnabled()) {
					log.debug("Original error: " + e, e);
			}
		}
	}
			
	}

	/**
	 * handles the incoming provider messages
	 * 
	 * @param exchange
	 * @throws MessagingException
	 */
	private void wait(MessageExchange exchange) throws MessagingException {
		// Retrieve correlation id
		log.info("WaitComponent->wait - Start");
		
		String correlationId = (String) exchange
				.getProperty(JbiConstants.CORRELATION_ID);
		
		String spagicCorrelationID = "SMX:"+correlationId;
		log.info("WaitComponent->wait - Retrieved Correlation ID " + correlationId);
		if (correlationId == null) {
			log
					.warn("WaitComponent->wait - Process Correlation ID not present in exchange: "
							+ exchange);
			return;
		}

		saveWaitData(spagicCorrelationID, exchange);
		
		// In the case of we've received an InOnly or RobustInOnly we must send
		// the done
		if (isInOnly(exchange) || isRobustInOnly(exchange)){
			log.info("WaitComponent->wait - Exchane is InOnly or RobustInOnly sending done");
			done(exchange);
		}
		
		log.info("WaitComponent->wait - End");
		// In the case we've received an InOut we must do nothing
		return;
	}

	protected void saveWaitData(String correlationId, MessageExchange exchange) {
		log.info("WaitComponent->saveWaitData - Start");
		Session aSession = null;
		Transaction tx = null;
		try {
			
			log.info("WaitComponent->saveWaitData - Getting Session from Hibernate");
			aSession = HibernateUtil.getSessionFactory().openSession();
			tx = aSession.beginTransaction();
			
			NormalizedMessage in = exchange.getMessage("in");
			String payloadOfNormalizedMessage = new SourceTransformer().contentToString(in);
			
			byte[] exchangeAsByteArray = serializeExchange(exchange);
			
			DateTime now = new DateTime();
			WaitData wd = new WaitData(correlationId, 
									  now.toDate(),
									   exchangeAsByteArray, 
									   this.signalEndpointURI.toString(), 
									   this.destinationEndpointURI.toString());
			if (this.eventType != null)
				wd.setEventType(this.eventType);
			else
				wd.setEventType(IEvent.GENERIC_EVENT_TYPE);
			
			if ((this.expirationPolicy != null) && !(this.expirationPolicy.equalsIgnoreCase(EXPIRATION_POLICY_NEVER_EXPIRE))){
				wd.setExpirationAction(this.expirationAction);
				wd.setExpiryTimestamp(computeExpirationTimeStamp(now));
			}
			
			aSession.save(wd);
			aSession.flush();
			
			
			Document payloadDocument = DocumentHelper.parseText(payloadOfNormalizedMessage);
			XPath xPath = null; 
			log.info("WaitComponent->saving Wait Attributes Extracted from Normalized Message");
			for ( WaitDataRule wdRule : waitDataRules){
				if (isValidString(wdRule.getExtractionRule()))
					xPath = new DefaultXPath(wdRule.getExtractionRule());
					xPath.setNamespaceURIs(getNamespaceMap());
					Object evaluated = null;
					String evaluatedString = null;
					evaluated = xPath.valueOf(payloadDocument);
					if (evaluated instanceof String) {
						evaluatedString = (String)evaluated;
					}else  if (evaluated instanceof Node) {
						evaluatedString = ((Node)evaluated).asXML();
					}
					if (isValidString(evaluatedString)){
						log.info("WaitComponent->Wait Attribute["+wdRule.getName()+"] Value is ["+evaluatedString+"]");
						WaitDataAttribute wdAttribute = new WaitDataAttribute(wd,wdRule.getName(), evaluatedString);
						aSession.save(wdAttribute);
						aSession.flush();
					}else{
						log.warn("WaitComponent->Wait Attribute["+wdRule.getName()+"] exracted with ["+wdRule.getExtractionRule()+"] produce a null or empty string value is not saved");
					}
				}
			
			tx.commit();
			
			log.info("WaitComponent->saveWaitData - Saved Data Transaction Committed");
			
		} catch (Exception e) {

			log.error("WaitComponent->saveWaitData - Has Produced an exception", e);
			log.error("WaitComponent->saveWaitData - Rollback transition");
			if (tx != null)
				tx.rollback();

		} finally {
			
			log.info("WaitComponent->saveWaitData - Close Session");
			if ((aSession != null) && aSession.isOpen()) {
				aSession.close();
			}
			
			log.info("WaitComponent->saveWaitData - End");
		}
	}

	

	
	public Date computeExpirationTimeStamp(DateTime nowDateTime){
		if (this.expirationPolicy.equalsIgnoreCase(EXPIRATION_POLICY_AT_FIXED_DATE_TIME)){
			return dateTimefmt.parseDateTime(this.fixedExpirationDateTime).toDate();
		}if (this.expirationPolicy.equalsIgnoreCase(EXPIRATION_POLICY_AFTER_DATE_TIME_INTERVAL)){
			
			DateTime resultDate = null;
			int unitTimeToAdd = Integer.valueOf(this.expirationAfter);
			log.debug("computeExpirationTimeStamp->Adding ["+unitTimeToAdd+"] ["+UNIT_INTERVAL_DAY+"]");
			
			if (this.intervalDateTimeUnit.equalsIgnoreCase(UNIT_INTERVAL_DAY)){
				resultDate = nowDateTime.plusDays(unitTimeToAdd);
			}else if (this.intervalDateTimeUnit.equalsIgnoreCase(UNIT_INTERVAL_MONTH)){
				resultDate = nowDateTime.plusMonths(unitTimeToAdd);
			}else if (this.intervalDateTimeUnit.equalsIgnoreCase(UNIT_INTERVAL_YEAR)){
				resultDate = nowDateTime.plusYears(unitTimeToAdd);
			}else if (this.intervalDateTimeUnit.equalsIgnoreCase(UNIT_INTERVAL_HOUR)){
				resultDate = nowDateTime.plusHours(unitTimeToAdd);
			}else if (this.intervalDateTimeUnit.equalsIgnoreCase(UNIT_INTERVAL_MINUTE)){
				resultDate = nowDateTime.plusMinutes(unitTimeToAdd);
			}else if (this.intervalDateTimeUnit.equalsIgnoreCase(UNIT_INTERVAL_SECOND)){
				resultDate = nowDateTime.plusSeconds(unitTimeToAdd);
			}
			return resultDate.toDate();
		}else{
			return null;
		}
	}
	

	public QName getDestinationEndpointURI() {
		return destinationEndpointURI;
	}

	public void setDestinationEndpointURI(QName destinationEndpointURI) {
		this.destinationEndpointURI = destinationEndpointURI;
	}
	
	public QName getSignalEndpointURI() {
		return signalEndpointURI;
	}

	public void setSignalEndpointURI(QName signalEndpointURI) {
		this.signalEndpointURI = signalEndpointURI;
	}

	public static String retrieveWaitDataOnMessage(){
		return "";
	}

	public List<WaitDataRule> getWaitDataRules() {
		return waitDataRules;
	}

	public void setWaitDataRules(List<WaitDataRule> waitDataRules) {
		this.waitDataRules = waitDataRules;
	}

	public String getEventType() {
		return eventType;
	}

	public void setEventType(String eventType) {
		this.eventType = eventType;
	}

	public String getExpirationPolicy() {
		return expirationPolicy;
	}

	public void setExpirationPolicy(String expirationPolicy) {
		this.expirationPolicy = expirationPolicy;
	}

	public String getFixedExpirationDateTime() {
		return fixedExpirationDateTime;
	}

	public void setFixedExpirationDateTime(String fixedExpirationDateTime) {
		this.fixedExpirationDateTime = fixedExpirationDateTime;
	}

	public String getExpirationAfter() {
		return expirationAfter;
	}

	public void setExpirationAfter(String expirationAfter) {
		this.expirationAfter = expirationAfter;
	}

	public String getIntervalDateTimeUnit() {
		return intervalDateTimeUnit;
	}

	public void setIntervalDateTimeUnit(String intervalDateTimeUnit) {
		this.intervalDateTimeUnit = intervalDateTimeUnit;
	}

	public String getExpirationAction() {
		return expirationAction;
	}

	public void setExpirationAction(String expirationAction) {
		this.expirationAction = expirationAction;
	}
	


}
