package org.spagic.smx.components.wait;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import javax.annotation.Resource;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.ExchangeStatus;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.namespace.QName;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.JbiConstants;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.Transaction;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.spagic.event.model.BaseEvent;
import org.spagic.event.model.FileEvent;
import org.spagic.event.model.IEvent;
import org.spagic.metadb.base.ProcessInstanceStateConstants;
import org.spagic.metadb.dbutils.AuditDBUtils;
import org.spagic.metadb.dbutils.HibernateUtil;
import org.spagic.metadb.model.ProcessInstance;
import org.spagic.metadb.model.WaitData;

public class WrapToEvent extends EventBeanComponent implements MessageExchangeListener {
	
	private static final Log log = LogFactory.getLog(WrapToEvent.class);
	
	@Resource
	private DeliveryChannel channel;
	
	private String eventType = null;
	private  QName eventMatcherURI = null;
	
	public void onMessageExchange(MessageExchange exchange)
			throws MessagingException {
		if (exchange.getStatus() == ExchangeStatus.ACTIVE) {
			try{
				wrapEvent(exchange);
				if ( isInAndOut(exchange) ){
					responseOk(exchange);
				}else{
					done(exchange);
				}
			}catch (Exception e) {
				try{
					fail(exchange, e);
				} catch (Exception e2) {
					log.warn("Unable to handle error: " + e2, e2);
					if (logger.isDebugEnabled()) {
						log.debug("Original error: " + e, e);
					}
				}
			}
		}else{
        	return;
        }		
	}
	
	
	/**
	 * handles the incoming provider messages
	 * 
	 * @param exchange
	 * @throws MessagingException
	 */
	protected void wrapEvent(MessageExchange exchange)
			throws MessagingException, Exception  {
		
		log.info("wrapEvent -> Start");
		
		NormalizedMessage normalizedMessage = exchange.getMessage("in");
		
		String xmlString = new SourceTransformer().contentToString(normalizedMessage);
			
		log.info("wrapEvent -> normalized message");
		Document doc = DocumentHelper.parseText(xmlString);
		log.info("***************************");
		xmlString = doc.getRootElement().asXML();
		log.info("***************************");
		MessageExchange eventExchange = null;
		if (isInOnly(exchange)) {
			eventExchange = channel.createExchangeFactory().createInOnlyExchange();
		}else if (isRobustInOnly(exchange)){
			log.info("wrapEvent -> Retrieved Exchange was an RobustInOnly -> Need to Create another exchange");
			eventExchange = channel.createExchangeFactory().createRobustInOnlyExchange();
		}
		
		
			
		IEvent event = new BaseEvent();
		event.setEventType(getEventType());
		event.setXmlPayload(xmlString);		
		
		NormalizedMessage newMessage = eventExchange.createMessage();
		populateMessage(newMessage, event.asXML());
		
		eventExchange.setService(eventMatcherURI);
		eventExchange.setMessage(newMessage, "in");
			
		log.info("wrapEvent -> - InOnly send");
		channel.send(eventExchange);
	}		
	
	public String getEventType() {
		return eventType;
	}

	public void setEventType(String eventType) {
		this.eventType = eventType;
	}
	public void responseOk(MessageExchange exchange) throws MessagingException {
		NormalizedMessage out = exchange.createMessage();
		String xmlPayload = "<spagicevent:SignalResponse xmlns:spagicevent=\"urn:it:eng:spagic:event\">OK</spagicevent:SignalResponse>";
		out.setContent(new StringSource(xmlPayload));
		channel.send(exchange);
	}


	public QName getEventMatcherURI() {
		return eventMatcherURI;
	}


	public void setEventMatcherURI(QName eventMatcherURI) {
		this.eventMatcherURI = eventMatcherURI;
	}



}
