package org.spagic.smx.marshallers;

import java.text.SimpleDateFormat;
import java.util.Iterator;
import java.util.Map;

import javax.jbi.messaging.MessagingException;
import javax.jbi.messaging.NormalizedMessage;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.dom.DOMSource;

import org.apache.servicemix.components.util.MarshalerSupport;
import org.apache.servicemix.jbi.util.DOMUtil;
import org.apache.servicemix.quartz.support.QuartzMarshaler;
import org.apache.servicemix.quartz.support.ServiceMixJob;
import org.quartz.JobDataMap;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * The default implementation of the Quartz marshaler
 *
 * @version $Revision: 426415 $
 */
public class SpagicQuartzMarshaller extends MarshalerSupport implements QuartzMarshaler {
	
	private static final SimpleDateFormat quartzDateTimeFormatter = new SimpleDateFormat("dd-MM-yyyy HH:mm:ss z");
	 
	 public void populateNormalizedMessage(NormalizedMessage message, JobExecutionContext context) 
     throws JobExecutionException, MessagingException {

     JobDetail detail = context.getJobDetail();
     JobDataMap dataMap = detail.getJobDataMap();
     for (Iterator iter = dataMap.entrySet().iterator(); iter.hasNext();) {
         Map.Entry entry = (Map.Entry) iter.next();
         String key = (String) entry.getKey();
         if (!key.equals(ServiceMixJob.COMPONENT_NAME) && !key.equals(ServiceMixJob.ENDPOINT_NAME)) {
             Object value = entry.getValue();
             message.setProperty(key, value);
         }
     }
     try {
         Document document = getTransformer().createDocument();
         Element root = document.createElement("timer");
         document.appendChild(root);
         DOMUtil.addChildElement(root, "name", detail.getName());
         DOMUtil.addChildElement(root, "group", detail.getGroup());
         DOMUtil.addChildElement(root, "fullname", detail.getFullName());
         DOMUtil.addChildElement(root, "description", detail.getDescription());
         DOMUtil.addChildElement(root, "fireTime", quartzDateTimeFormatter.format(context.getFireTime()));
         message.setContent(new DOMSource(document));
     } catch (ParserConfigurationException e) {
         throw new MessagingException("Failed to create content: " + e, e);
     }
 }

}
